Index: trunk/Ohana/src/opihi/lib.data/page.c
===================================================================
--- trunk/Ohana/src/opihi/lib.data/page.c	(revision 10997)
+++ trunk/Ohana/src/opihi/lib.data/page.c	(revision 11007)
@@ -24,5 +24,5 @@
 }
 
 /* return the given book */
 Page *GetPage (Book *book, int where) {
 
@@ -33,4 +33,46 @@
   if (where >= book[0].Npages) return NULL;
   return (book[0].pages[where]);
+}
+
+/* return the given page with key restrictions */
+Page *GetPageRestricted (Book *book, int where, char *keyName, char *keyValue) {
+
+  int i;
+  int N, Nout;
+  char *value;
+
+  if (where < 0) where += book[0].Npages;
+  if (where < 0) return NULL;
+  if (where >= book[0].Npages) return NULL;
+
+  if (where >= 0) {
+    N = -1;
+    for (i = 0; (i < book[0].Npages) && (N < where); i++) {
+      value = BookGetWord (book[0].pages[i], keyName);
+      if ((value == NULL) && !strcmp (keyValue, "NULL")) {
+	N++;
+	Nout = i;
+      } 
+      if ((value != NULL) && !strcmp (keyValue, value)) {
+	N++;
+	Nout = i;
+      }
+    }
+  } else {
+    N = 0;
+    for (i = book[0].Npages - 1; (i >= 0) && (N > where); i--) {
+      value = BookGetWord (book[0].pages[i], keyName);
+      if ((value == NULL) && !strcmp (keyValue, "NULL")) {
+	N--;
+      } 
+      if ((value != NULL) && !strcmp (keyValue, value)) {
+	N--;
+      }
+    }
+  }
+
+  if (N != where) return NULL;
+
+  return (book[0].pages[Nout]);
 }
 
@@ -24,5 +24,5 @@
 }
 
-/* return the given book */
+/* return the given page */
 Page *GetPage (Book *book, int where) {
 
